@@ -33,7 +33,7 @@ use Storable 'dclone';
 
 
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use constant NO_BINNING   => 0;
 use constant LIN_BINNING  => 1;
@@ -127,6 +127,44 @@ sub add_data_from_file {
   return;
 }
 
+sub add_data_from_frequencyfile {
+  my $self = shift;
+  my $filename = shift;
+  my @cols;
+  my $f; 
+  my $line;
+  open($f, "<", $filename) or die "Cannot open " .$filename . ": $!";
+  while($line = <$f>) {
+    chomp($line);
+    if($line =~ /^\s*#/) {
+      next;
+    }
+    @cols = split /\s+/ , $line;
+    #DEBUG print $line . "\t" . @cols . "\n";
+    if(scalar @cols == 2) {   # assume list of value - count
+      if(defined($self->{"data_frequency"}{$cols[0]})) {
+	$self->{"data_frequency"}{$cols[0]} += $cols[1];
+      }
+      else {
+	$self->{"data_frequency"}{$cols[0]} = $cols[1];
+      }
+      next;
+    }
+    # File format not recognized
+    if(scalar @cols != 2){
+      print "File format not recognized\n";
+      close($f);
+      delete $self->{"stats"};
+      delete $self->{"bins"};
+      return;      
+    }
+  }
+  close($f);
+  # data has changed - stats and bins need to be computed again
+  delete $self->{"stats"};
+  delete $self->{"bins"};
+  return;
+}
 
 
 ################## Descriptive Statistics ##################
@@ -172,7 +210,7 @@ sub count {
   if(!defined($self->{"stats"}{"Desc"}{"count"})) {
     my $count = 0;
     my $data;
-    foreach $data(keys $self->{"data_frequency"}) {
+    foreach $data(keys %{$self->{"data_frequency"}}) {
       $count += $self->{"data_frequency"}{$data};
     }
     $self->{"stats"}{"Desc"}{"count"} = $count;
@@ -190,7 +228,7 @@ sub mean {
     my $cumul_value = 0;
     my $count = $self->count();
     my $v;
-    foreach $v( keys $self->{"data_frequency"}) {
+    foreach $v( keys %{$self->{"data_frequency"}}) {
       $cumul_value += $v * $self->{"data_frequency"}{$v};
     }
     if($count > 0) {
@@ -221,7 +259,7 @@ sub median {
     my $current_index = 0;
 
     my ($v,$i);
-    foreach $v(sort {$a<=>$b} keys $self->{"data_frequency"}) {
+    foreach $v(sort {$a<=>$b} keys %{$self->{"data_frequency"}}) {
       for($i=0; $i < $self->{"data_frequency"}{$v}; $i++) {
 	$current_index++;
 	if($current_index == $median_index) {
@@ -278,7 +316,7 @@ sub variance {
     my $cumul_value = 0;    
     my $square_mean = 0;
     my $v;
-    foreach $v(keys $self->{"data_frequency"}) {
+    foreach $v(keys %{$self->{"data_frequency"}}) {
       $cumul_value += ($v**2) * $self->{"data_frequency"}{$v};
     }
     if($count > 0) {
@@ -312,6 +350,10 @@ sub standard_deviation {
 #  of values within a particular group or interval
 sub frequency_distribution {
   my $self = shift;
+  if($self->count() == 0) {
+    print "WARNING: empty dataset, returning empty frequency distribution. \n"; 
+    return (); # empty hash
+  }
   if($self->{"bin-type"} == NO_BINNING) { 
     # $self->{"stats"}{"Dist"}{"frequency"}
     return $self->_frequency_distribution();
@@ -329,6 +371,10 @@ sub frequency_distribution {
 #  equal to some value.
 sub probability_mass_function {
   my $self = shift;
+  if($self->count() == 0) {
+    print "WARNING: empty dataset, returning empty probability mass function. \n"; 
+    return (); # empty hash
+  }
   if($self->{"bin-type"} == NO_BINNING) { 
     # $self->{"stats"}{"Dist"}{"pmf"}
     return $self->_probability_mass_function();
@@ -347,6 +393,10 @@ sub probability_mass_function {
 #  by 1/n at each of the n data points.
 sub empirical_distribution_function {
   my $self = shift;
+  if($self->count() == 0) {
+    print "WARNING: empty dataset, returning empty emptirical distribution function. \n"; 
+    return (); # empty hash
+  }
   if($self->{"bin-type"} == NO_BINNING) { 
     # $self->{"stats"}{"Dist"}{"cdf"}
     return $self->_empirical_distribution_function();
@@ -362,6 +412,10 @@ sub empirical_distribution_function {
 #  how often the random variable is above a particular level.
 sub complementary_cumulative_distribution_function  {
   my $self = shift;
+    if($self->count() == 0) {
+    print "WARNING: empty dataset, returning empty complementary cumulative distribution function. \n"; 
+    return (); # empty hash
+  }
   if($self->{"bin-type"} == NO_BINNING) { 
     # $self->{"stats"}{"Dist"}{"ccdf"}
     return $self->_complementary_cumulative_distribution_function();
@@ -593,14 +647,18 @@ sub set_custom_num_bins {
 
 sub bins {
   my $self = shift;
-  if(!defined($self->{"bins"})) {
-    
+  if($self->count() == 0) {
+    print "WARNING: empty dataset, returning empty bins. \n"; 
+    return {}; # empty array
+  }
+
+  if(!defined($self->{"bins"})) {    
     # CASE 1: if no_binning is set we do not
     # save the binning structure
     if( $self->{"bin-type"} == NO_BINNING) {
       my $curbins = {};
       my $v;
-      foreach $v(keys $self->{"data_frequency"}) {
+      foreach $v(keys %{$self->{"data_frequency"}}) {
 	my $ref = $v;
 	$curbins->{$v}{"left"} = $v;
 	$curbins->{$v}{"right"} = $v;
@@ -628,7 +686,7 @@ sub bins {
 	print "using the NO_BINNING mode" . "\n";
 	my $curbins = {};
 	my $v;
-	foreach $v(keys $self->{"data_frequency"}) {
+	foreach $v(keys %{$self->{"data_frequency"}}) {
 	  my $ref = $v;
 	  $curbins->{$v}{"left"} = $v;
 	  $curbins->{$v}{"right"} = $v;
@@ -773,7 +831,7 @@ sub _full_support {
   my $self = shift;
   my $support = {};
   my $v;
-  foreach $v( keys $self->{"data_frequency"}) {
+  foreach $v( keys %{$self->{"data_frequency"}}) {
     $support->{$v} = 1;
   }
   return (keys %{$support});
@@ -813,6 +871,14 @@ sub compute_log_bins {
   my $num_bins = shift;
   my $min = $self->minimum();
   my $max = $self->maximum();
+  if($min == 0 or $max == 0 or $min *$max <0) {
+    # if the interval contains zero, then we cannot logbin
+    # we return the linear binning
+    print "Logarithmic binning was not possible on this sample, ";
+    print "switching to linear binning mode" . "\n";
+    $self->{"bin-type"} = LIN_BINNING;
+    return $self->compute_lin_bins($num_bins);    
+  }
   my $logbins = {}; # reference to empty hash
   my $min_exp = log($min)/log(LOG_BASE);
   my $max_exp = log($max+1)/log(LOG_BASE);
